# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module cointklib.dialogs
##   Ce module contient des dfinitions de botes de dialogue
##   gnriques utilises par l'interface de pycc
## ----------------------------------------------------------------------

import sys
sys.path.append('..')

from coinlib.baseclasses import PyccError
from basicwidgets import *
#from widgets import OptionsEditor
from constants import *
import tkMessageBox

class CoinTopLevel(Toplevel):
  
  def __init__(self, parent, title=None, buttons=None):
    Toplevel.__init__(self, parent)
    if title:
      self.title(title)
    self.parent = parent
    self.initial_focus = ''
    if buttons:
      body = Frame(self, border=2, relief=RAISED)
      self.initial_focus = self.body(body)
      body.pack(fill=BOTH, expand=1)
      self.buttonbox(buttons)
    self.protocol("WM_DELETE_WINDOW", self.cancel)
  
  def body(self, master):
    pass # abstract
  
  def buttonbox(self, buttons):
    box = Frame(self)
    for but in buttons:
      w = Button(box, text=but, width=10, command=self.get_callback(but))
      w.pack(side=LEFT, padx=5)
      if but == BUT_OK:
        w.config(default=ACTIVE)
    self.bind("<Return>", self.ok)
    self.bind("<Escape>", self.cancel)
    box.pack(pady=5)

  def get_callback(self, button):
    if button == BUT_OK:
      return self.ok
    elif button == BUT_CANCEL or button == BUT_CLOSE:
      return self.cancel
    elif button == BUT_APPLY:
      return self.apply
    elif button == BUT_SAVE:
      return self.save
    elif button == BUT_NEW:
      return self.new
    elif button == BUT_DEL:
      return self.delete
    else:
      return self.bind_button(button)
  
  def bind_button(self, button):
    # Mthode abstraite, par dfaut retourne une fonction nulle
    return lambda w=None, e=None: None
  
  def ok(self, event=None):
    self.withdraw()
    self.update_idletasks()
    self.apply()
    self.parent.canceled = 0
    self.cancel()
  
  def cancel(self, event=None):
    self.destroy()
  
  def apply(self, event=None):
    pass # abstract
  
  def new(self, event=None):
    pass # abstract
  
  def delete(self, event=None):
    pass # abstract
  
  def save(self, event=None):
    pass # abstract


class CoinDialog(CoinTopLevel):

  def __init__(self, parent, title=None, buttons=[ BUT_OK, BUT_CANCEL ], modal=0):
    CoinTopLevel.__init__(self, parent, title, buttons)
    self.transient(parent)
    self.result = None
    self.callbacks = {}
    if modal: self.grab_set()
    if not self.initial_focus:
      self.initial_focus = self
    if self.parent is not None:
      self.geometry("+%d+%d" % (parent.winfo_rootx()+50, parent.winfo_rooty()+50))
    self.initial_focus.focus_set()
    self.wait_window(self)
  
  def ok(self, event=None):
    if not self.validate():
      self.initial_focus.focus_set()
      return
    self.withdraw()
    self.update_idletasks()
    self.apply()
    self.parent.canceled = 0
    self.cancel()
  
  def cancel(self, event=None):
    if self.parent is not None:
      self.parent.focus_set()
    self.destroy()
  
  def validate(self):
    return 1 # abstract
  

class SelectDialog(CoinDialog):

  def __init__(self, parent, items, title=None, text='', **kwargs):
    self.items = items
    self.text = text
    self.kwargs = kwargs
    CoinDialog.__init__(self, parent, title)
  
  def body(self, master):
    Label(master, text=self.text).pack(side=TOP, padx=6)
    self.list = SortedListBox(master, self.items, **self.kwargs)
    self.list.pack(side=TOP, padx=6, pady=6, fill=BOTH, expand=1)
  
  def apply(self):
    self.parent.selected = self.list.selected
  

class OptionsDialog(CoinDialog):
  
  def __init__(self, title, parent, object, extra_arg):
    self.object = object
    self.extra_arg = extra_arg
    CoinDialog.__init__(self, parent, title, [BUT_OK, BUT_CANCEL], modal=1)
  
  def body(self, master):
    from widgets import OptionsEditor
    self.w_options = OptionsEditor(master, self.object, extra_arg=self.extra_arg)
    self.w_options.pack()
  
  def apply(self):
    self.w_options.apply()
  

class LoginDialog(CoinDialog):
  
  def __init__(self, parent, pycc, trib=None, ident=None, force=0, **kwargs):
    self.pycc = pycc
    self.trib = trib
    self.ident = ident
    self.force = force
    CoinDialog.__init__(self, parent, **kwargs)
  
  def body(self, master):
    Label(master, text="Authentification sur un bouchot", background="#FCFCDA").pack(fill="x")
    self.w_login = TextField(master, "Login:")
    self.w_login.pack(anchor="e")
    self.w_passwd = TextField(master, "Password:")
    self.w_passwd.pack(anchor="e")
    fr = Frame(master)
    self.w_tribsel = ComboField(fr, "Bouchot:", self.pycc.bouchots.keys())
    self.w_tribsel.bind_onchange(self.change_trib)
    self.w_tribsel.pack(side="left", anchor="w")
    self.w_idsel = ComboField(fr, "Identit:", self.pycc.identities.get().keys())
    self.w_idsel.bind_onchange(self.change_id)
    self.w_idsel.pack(side="right", anchor="e")
    fr.pack(fill=X)
    if self.trib is None:
      self.trib = pycc.get_bouchot()
    if self.ident is None:
      self.ident = pycc.bouchots[self.trib].options.val('default_id')
    self.w_tribsel.set(self.trib)
    self.change_id(self.ident)
    if self.force:
      print "### AFR le state a marche paaaaas"
      #self.w_tribsel.field.configure(state="disabled")
      #self.w_idsel.field.configure(state="disabled")
  
  def change_id(self, ident=None):
    if ident is not None:
      self.w_idsel.set(ident)
    self.update_auth()
  
  def update_auth(self):
    ident = self.pycc.identities.get()[self.w_idsel.get()]
    login, passwd = ident.get_auth(self.w_tribsel.get())
    if login:
      self.w_login.set(login)
    elif not self.w_login.get():
      self.w_login.set(ident.login)
    if passwd:
      self.w_passwd.set(passwd)
  
  def change_trib(self, trib=None):
    if trib is not None:
      self.w_tribsel.set(trib)
    self.update_auth()
  
  def apply(self):
    bouchot = self.w_tribsel.get()
    trib = self.pycc.bouchots[bouchot].trib
    if trib is None:
      tkMessageBox.showinfo("pyCoinCoin - Login on bouchot", \
            "Login impossible, ce bouchot n'a pas de tribune", \
            parent=self)
      return
    ident = self.pycc.identities.get()[self.w_idsel.get()]
    login = self.w_login.get()
    passwd = self.w_passwd.get()
    try:
      cookie = trib.loginOnBouchot(login, passwd)
    except PyccError, reason:
      tkMessageBox.showinfo("pyCoinCoin - Login on bouchot", \
            "L'authentification a chou:\n%s" % reason, \
            parent=self)
      return
    if not cookie:
      tkMessageBox.showinfo("pyCoinCoin - Login on bouchot", \
            "L'authentification a chou:\npycc n'a pu choper aucun cookie.\nVrifiez vos paramtres, ptt...", \
            parent=self)
      return
    ident.set_cookie(bouchot, cookie)
    ident.save_auth(bouchot, login, passwd)
    self.pycc.palmi.set_id(ident.login)
    self.parent.result = (ident.login, bouchot)
  

class LogoutDialog(CoinDialog):
  
  def __init__(self, parent, pycc, trib=None, ident=None, **kwargs):
    self.pycc = pycc
    self.trib = trib
    self.ident = ident
    CoinDialog.__init__(self, parent, **kwargs)
  
  def body(self, master):
    Label(master, text="Se dloguer d'un bouchot", background="#FCFCDA").pack(fill="x")
    fr = Frame(master)
    self.w_tribsel = ComboField(fr, "Bouchot:", self.pycc.activeBouchots())
    self.w_tribsel.pack(side="left", anchor="w")
    self.w_idsel = ComboField(fr, "Identit:", self.pycc.identities.get().keys())
    self.w_idsel.pack(side="right", anchor="e")
    fr.pack(fill=X)
    if self.trib is None:
      self.trib = pycc.get_bouchot()
    if self.ident is None:
      self.ident = pycc.bouchots[self.trib].options.val('default_id')
    self.w_tribsel.set(self.trib)
    self.w_idsel.set(self.ident)
  
  def apply(self):
    bouchot = self.w_tribsel.get()
    trib = self.pycc.bouchots[bouchot].trib
    if trib is None:
      tkMessageBox.showinfo("pyCoinCoin - Logout from bouchot", \
            "Logout impossible, ce bouchot n'a pas de tribune", \
            parent=self)
      return
    ident = self.pycc.identities.get()[self.w_idsel.get()]
    res = trib.logoutFromBouchot(ident)
    if res:
      tkMessageBox.showinfo("pyCoinCoin - Logout from bouchot", res, parent=self)
      return
    self.pycc.palmi.set_id(ident.login)
    self.parent.result = (ident.login, bouchot)
