# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module multirss
##   Dfinition du module de bouchot MultiRSS :
##   monitoring de fils rss multiples dans une mme fentre
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import xml.dom.minidom
import tkMessageBox

from constants import *
from modules.module import *
from coinlib.persistants import XMLizable, OptionSet, Option, RSSItemOptions
from coinlib.utils import *
from coinlib.parsers import *
from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor
from cointklib.dialogs import OptionsDialog

def build_module(parent, name):
  return MultiRSS(parent, name)

class MultiRSSOptions(OptionSet):
  """ Options du module MultiRSS
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "MultiRSS"),
      'short_name':   Option("Nom court (pour la diode)", "MultiRSS"),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'check_delay':  Option("Rafrachir toutes les (en secondes)", 600, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 200, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'rss_list':     Option("Liste des fils RSS", [RSSItemOptions()], TYPE_LIST),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'check_delay', 'max_history', 'read_at_exp', 'docked', 'display_img' ],
          "Fils RSS":[ 'rss_list' ]
        } )
    }


class MultiRSSEditor(OptionsEditor):
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    self.widgets[name] = OrderedListBoxWithButtons(fr, option.get(), 1, width=40, height=10)
    self.widgets[name].pack(fill=X)
    self.widgets[name].bind_new(self.new)
    self.widgets[name].bind_edit(self.edit)
  
  def new(self):
    self.canceled = 1
    newfeed = RSSItemOptions()
    OptionsDialog("Nouveau fil RSS", self, newfeed, self.extra_arg)
    if not self.canceled:
      return newfeed
  
  def edit(self):
    rssitem = self.widgets['rss_list'].get(self.widgets['rss_list'].selected)
    OptionsDialog("Proprits du fil RSS", self, rssitem, self.extra_arg)
    # self.w_actions.refresh()
    return 1
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'rss_list':
        self.object.set(opt, widget.getAll())
      else:
        self.object.set(opt, widget.get())
    self.object.save()


class MultiRSSItem(object):
  
  def __init__(self, attrs={}):
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read     = 0   # Flag lu/nonlu
    self.expanded = 0   # Flag description ouverte/ferme
    self.next = None    # Pointeur sur le suivant
    self.feedopts = None
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def getFeedName(self):
    if not self.attributes.has_key('feedname'):
      return u'[pas de nom]'
    return "["+self.attributes['feedname']+"]"
  
  def getTitle(self):
    if not self.attributes.has_key('title'):
      return u'[pas de titre]'
    return self.attributes['title']
  
  def getLink(self):
    for lkey in ['link', 'guid']:
      if self.attributes.has_key(lkey):
        return self.attributes[lkey]
      else:
        continue
    return ''
  
  def getDate(self):
    for dkey in ['date', 'pubDate', 'dc:date']:
      if self.attributes.has_key(dkey):
        return self.attributes[dkey]
      else:
        continue
    return ''
  
  def getAuthor(self):
    for akey in ['author', 'creator', 'dc:creator']:
      if self.attributes.has_key(akey):
        return self.attributes[akey]
      else:
        continue
    return ''
  
  def getDescr(self):
    res = self.getContent()
    if not res:
      try:
        res = self.attributes['description']
      except:
        res = ''
    return res
  
  def getContent(self):
    for ckey in ['content:encoded', 'content']:
      if self.attributes.has_key(ckey):
        return self.attributes[ckey]
      else:
        continue
    return ''
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class MultiRSSViewer(GenericViewer):

  def __init__(self, parent, module):
    self.cur_opts = RSSItemOptions()
    GenericViewer.__init__(self, parent, module)
    """
    self.root = parent
    self.multirss = multirss
    self.styles = self.multirss.bouchot.styles
    self.cur_opts = RSSItemOptions()
    Frame.__init__(self, parent)
    self.build_interface()
    self.update()
    """
  
  def build_interface(self):
    GenericViewer.build_interface(self)
    fr = Frame(self)
    self.feedname = TextField(fr, "Nom:", width=12)
    self.feedname.pack(side=LEFT)
    self.feedurl = TextField(fr, "URL:", width=42)
    self.feedurl.pack(side=LEFT, fill=X)
    Button(fr, text="Ajouter", border=1, command=self.addRSS).pack(side=LEFT, fill=X)
    Button(fr, text="Options", border=1, command=self.showRSSOpts).pack(side=LEFT, fill=X)
    fr.pack(side=TOP, fill=X)
  
  def show_menu(self, event):
    """
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      cur_rss = [rss for rss in self.multirss.options.val('rss_list') if rss.val('name') == t]
      if cur_rss:
        self.menu.add_command("Config du rss courant")
    # AFR: c'est juste l'ide
    """
    self.menu.post(event.x_root, event.y_root)
  
  def showRSSOpts(self):
    self.cur_opts.set('name', self.feedname.get())
    self.cur_opts.set('backend', self.feedurl.get())
    OptionsDialog("Proprits du fil RSS", self, self.cur_opts, self.module.bouchot.pycc)
  
  def addRSS(self):
    feedname = self.feedname.get()
    feedurl = self.feedurl.get()
    if not feedname or not feedurl:
      return
    rsslist = self.module.options.val('rss_list')
    if feedname in [ rss.val('name') for rss in rsslist ]:
      tkMessageBox.showwarning('MultiRSS', "Un fil de nom %s existe dj" % feedname, parent=self)
      return
    if feedurl in [ rss.val('backend') for rss in rsslist ]:
      tkMessageBox.showwarning('MultiRSS', "Un fil d'url %s existe dj" % feedurl, parent=self)
      return
    newrss = self.cur_opts
    self.cur_opts = RSSItemOptions()
    newrss.set('name', feedname)
    newrss.set('backend', feedurl)
    self.module.options.set('rss_list', rsslist+[newrss])
    self.module.options.save()
    self.module.thread.restart()
    self.apply_styles()
  
  def apply_styles(self):
    apply_styles(self.list, self.styles, 'pinni', self.module.bouchot.pycc.default_styles)
    self.list.tag_config('descr', lmargin1=20, lmargin2=20, borderwidth=1, relief=FLAT)
    for rssopt in self.module.options.val('rss_list'):
      self.list.tag_config(rssopt.val('name'), background=rssopt.val('color'))
      self.list.tag_raise(rssopt.val('name'))
  
  def insertItem(self, item):
    self.addFoldSign(item)
    self.list.config(state=NORMAL)
    self.list.insert(INSERT, item.getFeedName(), item.feedopts.val('name'))
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.getDate(), "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.getAuthor(), "login")
    self.list.insert(INSERT, " ")
    t = "id%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    self.module.bouchot.pycc.addUrlInList(item.getLink(), item.getDate()+"@"+self.module.name+":"+item.feedopts.val('name'))
    if item.expanded:
      text = toUnicode(item.getDescr(), item.feedopts.val('encoding'))
      msg, tags = self.module.parseDescr(text, item.feedopts)
      # self.module.bouchot.pycc.addUrls([tag[4:] for tag in tags if tag[:4]=="url_"], item.getDate()+"@"+self.module.name+":"+item.feedopts.val('name'))
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      self.addTags(tags, start)
    end_pos = self.list.index("insert")
    #self.list.tag_add(item.feedopts.val('name'), orig_pos, end_pos)
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
  

def getText(nodelist, encoding='iso-8859-15'):
  rc = ""
  for node in nodelist:
    if node.nodeType == node.TEXT_NODE:
      rc = rc + node.data
  return toUnicode(rc, encoding).strip()

def getXML(nodelist, encoding='iso-8859-15'):
  rc = ""
  for node in nodelist:
    rc = rc + node.toxml()
  rc = rc.replace('<![cdata[','')
  rc = rc.replace('<![CDATA[','')
  rc = rc.replace(']]>','')
  return toUnicode(rc, encoding).strip()


class MultiRSS(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
    self.rssElements = ['title', 'link', 'guid', 'description', 'pubDate', 'date', 'author', 'dc:date', 'dc:creator', 'content:encoded']
    self.openViewer()
    self.apply_styles()
  
  def config_frame(self, master):
    return MultiRSSEditor(master, self.options, self.bouchot.pycc)
  
  def build_viewer(self, parent):
    self.viewer = MultiRSSViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = MultiRSSOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def action(self):
    for rssitem in self.options.val('rss_list'):
      self.error = ""
      rssname = rssitem.val('name')
      rssurl = rssitem.val('backend')
      self.setState(STATE_HTTP)
      self.led.setLabel("["+rssname+"]")
      if rssurl[:7] == 'http://':
        rssurl = rssurl[7:]
      pos = rssurl.find('/')
      server = rssurl[:pos]
      url = rssurl[pos:]
      ua, cookie, data = self.get_http_params()
      headers = {}
      try:
        file = open_url(server, url, self.bouchot.pycc.options, ua, cookie, data, headers)
      except HTTPError, value:
        if value.code == 302 or value.code == 304:
          self.setState(STATE_OK)
          log.debug("%s - %s" % (url, value))
        else:
          self.error = str(value)
          self.setState(STATE_ERROR, self.error)
          log.error("Erreur HTTP sur %s:%s - %s:\n    %s" % (self.name, rssname, rssurl, str(value)))
      except URLError, value:
        self.error = "["+rssname+"] "+str(value.reason)
        self.setState(STATE_ERROR, self.error)
        log.error("%s:%s Erreur en essayant de rapatrier %s:\n    %s" % (self.name, rssname, rssurl, str(value.reason)))
      except:
        self.error = "%s: Y'a eu un truc zarb sur %s" % (rssname, rssurl)
        self.setState(STATE_ERROR, self.error)
        log.error("%s:%s Erreur http inconnue sur %s" % (self.name, rssname, rssurl))
      object_list = []
      if not self.error:
        try:
          object_list = self.parse(file, rssitem)
        except:
          self.error = "[fil %s] Ca a plant bizarrement" % rssname
          self.setState(STATE_ERROR, self.error)
          log.error("%s:%s Erreur inconnue en parsant %s" % (self.name, rssname, rssurl))
        else:
          if self.error:
            self.setState(STATE_WARNING, self.error)
            log.warning("Erreur de parsing sur %s:%s\n    %s" % (self.name, rssname, self.error))
          else:
            self.setState(STATE_OK)
        file.close()
      if object_list:
        self.process(object_list)  # Ce qu'on fait des objets parss
    self.led.setLabel(self.options.val('short_name'))
  
  def getEncoding(self, line, rssitem):
    if line.find("<?xml") >= 0:
      try:
        encoding = getAttr("encoding", line)
        rssitem.set("encoding", encoding)
        log.info("%s: Encoding dtect dans le backend: %s" % (self.name, encoding))
        return encoding
      except:
        pass
    encoding = rssitem.val('encoding')
    log.info("%s: Pas d'encoding dans le backend, valeur par dfaut: %s" % (self.name, encoding))
    return encoding
  
  def parse(self, file, rssitem):
    tmplist = []
    text = file.read()
    encoding = self.getEncoding(text, rssitem)
    # rssitem.set('encoding', encoding)
    try:
      dom = xml.dom.minidom.parseString(text)
    except:
      self.error = "[fil %s] Erreur de parsing XML (fichier rss moisi ?)" % rssitem.val('name')
      return []
    channel = dom.getElementsByTagName("channel")[0]
    title = getText(channel.getElementsByTagName("title")[0].childNodes, encoding)
    if not rssitem.val('keep_cr'):
      title = title.replace('\n', ' ')
    rssitem.set('descr', title)
    for item in dom.getElementsByTagName("item"):
      attrs = {'feedname':rssitem.val('name')}
      for elem in self.rssElements:
        domlist = item.getElementsByTagName(elem)
        if len(domlist):
          if elem in ['description', 'title', 'content:encoded']:
            msg = getXML(domlist[0].childNodes, encoding)
            attrs[elem] = msg
          else:
            attrs[elem] = getText(domlist[0].childNodes, encoding)
            if elem == 'link':
              link = attrs[elem]
          if not rssitem.val('keep_cr'):
            attrs[elem] = attrs[elem].replace('\n', ' ')
      newitem = MultiRSSItem(attrs)
      newitem.feedopts = rssitem
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
    dom.unlink()
    tmplist.reverse()
    return tmplist
  
  def process(self, items=[]):
    for rssitem in items:
      if self.firstId is None:
        self.firstId = rssitem.id
      if rssitem.id not in self.objects.keys():
        log.debug("%s: nouvel rssitem: %s" % (self.name, rssitem.id))
        self.objects[rssitem.id] = rssitem
        if self.lastId is not None:
          self.objects[self.lastId].next = rssitem
        self.lastId = rssitem.id
        if self.viewer_open:
          self.viewer.insertItem(rssitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    if not item.read:
      try:
        f = open(os.path.join(self.config_path, "read"), 'a')
        f.write(item.id+"\n")
        f.close()
      except:
        log.warning("%s: Ecriture dans le fichier 'read' impossible" % self.name)
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
    self.bouchot.pycc.setUrlRead(item.getLink())
  
  def isRead(self, id):
    try:
      f = open(os.path.join(self.config_path, "read"), 'r')
      readlist = f.read().split("\n")
      f.close()
    except:
      log.warning("%s: Lecture dans le fichier 'read' impossible" % self.name)
      return 0
    return id in readlist
  
  def parseDescr(self, text, rssopt):
    if rssopt.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    text = htmlParser.pre_parse(text, slip_style, rssopt.val('encoding'))
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = get_url_from_base(rssopt.val('backend'), getAttr("src", text[start:end]))
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        self.bouchot.pycc.getImageWithCache(url)
    msg, tags = htmlParser.html_parse(text, encoding=rssopt.val('encoding'))
    return (msg, tags)
  