# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module userinfo
##   Dfinition du module de bouchot UserInfo :
##   monitoring des infos sur l'utilisateur (XPs, votes...)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import *

def build_module(parent, name):
  return UserInfo(parent, name)


class UserInfoOptions(OptionSet):
  """ Options du module UserInfo
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "User Info & Messagerie"),
      'short_name':   Option("Nom court (pour la diode)", ""),
      'backend':      Option("URL du fichier de backend", "/users/"),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'srv_type':     Option("Type de serveur", "templeet", TYPE_CHOICE),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'backend', 'check_delay', 'srv_type', 'max_history', 'docked' ]
        } )
    }
  

class MsgItem(object):
  
  def __init__(self, type, attrs={}):
    self.type = type
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read = 0     # Flag lu/nonlu
    self.next = None  # Pointeur sur le suivant
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def __str__(self):
    return repr(self.attributes['title'])
  
  def getTitle(self):
    return self.attributes['title']
  
  def getStatus(self):
    return self.attributes['status']
  
  def getLink(self):
    return self.attributes['link']
  
  def getDate(self):
    try:
      return self.attributes['date']
    except:
      return ""
  
  def getAuthor(self):
    return self.attributes['author']
  
  def getDescr(self):
    try:
      return self.attributes['descr']
    except:
      return ""
  
  def getNote(self):
    try:
      return self.attributes['note']
    except:
      return ""
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class UserViewer(GenericViewer):
  
  def build_menu(self):
    pass
  
  def build_interface(self):
    self._build_topframe()
    self._build_ctxmenu()
    self.tablist = { "Commentaires": None, "Messages reus": None, "Messages envoys": None }
    self.tabs = Pmw.NoteBook(self, raisecommand=self.tabselect, tabpos='n', borderwidth=1)
    self.tabs.pack(side="left", fill="both", expand=1, padx=0, pady=0)
    self.in_construction = 1
    for tab in self.tablist.keys():
      page = self.tabs.add(tab)
      self.tablist[tab] = ScrolledText(page, wrap=WORD)
      #self.list = self.tablist[tab]
      self.tablist[tab].pack(side=TOP, fill=BOTH, expand=1)
      self.tablist[tab].bind("<Enter>", lambda e, w=self: w.list.focus_set())
      self.tablist[tab].bind("<Motion>", self.mouse_move)
      self.tablist[tab].bind("<1>", self.mouse_click)
      self.tablist[tab].bind("<2>", self.mouse_click2)
      self.tablist[tab].bind("<3>", self.show_menu)
      self.tablist[tab].bind("<Escape>", self.hide_menu)
    self.tabs.setnaturalsize()
    self.in_construction = 0
    self.tabselect("Commentaires")
  
  def tabselect(self, tab):
    if self.in_construction: return
    self.list = self.tablist[tab]
    self.tabs.selectpage(tab)
    self.update()
  
  def parseDescr(self, text):
    text = htmlParser.pre_parse(text, "Tags not encoded")
    msg, tags = htmlParser.html_parse(text)
    return (msg, tags)

  def insertItem(self, item):
    if item.type != self.tabs.getcurselection():
      return
    self.list.config(state=NORMAL)
    if self.ascending.get():
      self.list.mark_set(INSERT, 0.0)
    else:
      self.list.mark_set(INSERT, END)
    if item.type == "Messages reus":
      self.list.insert(INSERT, item.getDate(), "time")
      self.list.insert(INSERT, " [%s] " % item.getStatus())
      self.list.insert(INSERT, item.getAuthor(), "login")
      self.list.insert(INSERT, " ")
      t = "id%s" % item.id
      self.list.insert(INSERT, item.getTitle(), t)
      self.list.insert(INSERT, "\n")
      self.setStyleTag(t, "a")
      self.setStyleTag(t, "url_"+item.getLink())
    elif item.type == "Messages envoys":
      t = "id%s" % item.id
      self.list.insert(INSERT, item.getTitle(), t)
      self.list.insert(INSERT, " ")
      self.list.insert(INSERT, item.getDescr())
      self.list.insert(INSERT, "\n")
      self.setStyleTag(t, "a")
      self.setStyleTag(t, "url_"+item.getLink())
    else:
      t = "id%s" % item.id
      self.list.insert(INSERT, item.getTitle(), t)
      lastrep = item.getDate()
      if lastrep:
        lastrep = " / derniere: " + lastrep
      self.list.insert(INSERT, " [%s reponses%s] <Note: " % (item.getStatus(), lastrep))
      self.list.insert(INSERT, item.getNote(), 'trollo')
      self.list.insert(INSERT, ">\n")
      self.setStyleTag(t, "a")
      self.setStyleTag(t, "url_"+item.getLink())
      text = toUnicode("Fil initiateur: "+item.getDescr(), 'iso-8859-15')
      text = htmlParser.pre_parse(text, "Tags not encoded")
      msg, tags = htmlParser.html_parse(text)
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      self.addTags(tags, start)
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
  

class UserInfo(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    #self.title = "Messagerie DLFP"
    self.title = "Infos utilisateur sur DLFP"
  
  def build_viewer(self, parent):
    # self.viewer = MsgViewer(parent, self)
    self.viewer = UserViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = UserInfoOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def get_http_params(self):
    idname = self.bouchot.options.val('default_id')
    id = self.bouchot.pycc.identities.get(idname)
    try:
      cookie = id.cookies[self.bouchot.name]
    except:
      cookie = ''
    return ('', cookie, None)
  
  def parse_votes(self, text):
    xp = "" # A pus les XPs...
    start = end = text.find("Avis :")
    if start >= 0:
      start += 7
      end = text.find("/", start)
      reste = text[start:end]
      start = end+1
      end = text.find("\n", start)
      votes = text[start:end]
    else:
      votes = reste = "?"
      end = 0
    return (xp, reste, votes)
  
  def parse_msg(self, text):
    tmplist = []
    end = 0
    start = text.find('<input type="checkbox" name="table[', end)
    while start > end:
      attrs = {}
      end = text.find('<td>', start)
      if end < start: break
      start = text.find('</td>', end)
      if start < end: break
      attrs['status'] = text[end+4:start].replace("\n", " ").strip()
      start = text.find('<a href="../~', end)
      if start < end: break
      end = text.find('/">', start)
      if end < start: break
      pseudo = text[start+13:end]
      start = text.find('</a>', end)
      if start < end: break
      attrs['author'] = text[end+3:start].replace("\n", " ").strip() + " (%s)" % pseudo
      end = text.find('<a href="viewmessage,', start)
      if end < start: break
      start = text.find('.html">', end)
      if start < end: break
      attrs['id'] = text[end+21:start]
      end = text.find('</a>', start)
      if end < start: break
      attrs['title'] = text[start+7:end].replace("\n", " ").strip()
      attrs['link'] = "http://%s%sviewmessage,%s.html" % (self.bouchot.options.val('http_server'), self.options.val('backend'), attrs['id'])
      newitem = MsgItem("Messages reus", attrs)
      newitem.read = self.isRead(newitem)
      # newitem.type = "Messages reus"
      tmplist.append(newitem)
      start = text.find('<input type="checkbox" name="table[', end)
    tmplist.reverse()
    return tmplist
  
  def parse_sentmsg(self, text):
    tmplist = []
    end = 0
    start = text.find('non lus par votre destinataire', end)
    if start > end:
      stop = text.find('</table>', start)
      end = start + 20
      start = text.find('<a href="../~', end)
      while stop > start:
        attrs = {}
        if start < end: break
        end = text.find('/">', start)
        if end < start: break
        pseudo = text[start+13:end]
        start = text.find('</a>', end)
        if start < end: break
        attrs['title'] = text[end+3:start].replace("\n", " ").strip() + " (%s)" % pseudo
        attrs['link'] = "http://%s/~%s/" % (self.bouchot.options.val('http_server'), pseudo)
        end = text.find('<td>', start)
        if end < start: break
        start = text.find('</td>', end)
        if start < end: break
        attrs['descr'] = text[end+4:start].replace("\n", " ").strip()
        newitem = MsgItem("Messages envoys", attrs)
        # newitem.read = self.isRead(newitem)
        tmplist.append(newitem)
        start = text.find('<a href="../~', end)
    tmplist.reverse()
    return tmplist
  
  def parse_comments(self, text):
    tmplist = []
    end = 0
    start = text.find('<h1>Derniers commentaires</h1>', end)
    if start > end:
      stop = text.find('</table>', start)
      end = start + 20
      start = text.find('<tr>', end)
      while stop > start:
        attrs = {}
        if start < end: break
        # Fil initiateur du commentaire
        end = text.find('<td>', start)
        if end < start: break
        start = text.find('</td>', end)
        if start < end: break
        attrs['descr'] = text[end+4:start].replace("\n", "").strip()
        # Titre et lien sur le commentaire
        end = text.find('<a href="../', start)
        if end < start: break
        start = text.find('">', end)
        if start < end: break
        attrs['link'] = "http://%s/%s" % (self.bouchot.options.val('http_server'), text[end+12:start])
        end = text.find('</a>', start)
        if end < start: break
        attrs['title'] = text[start+2:end].strip()
        # Note du commentaire
        start = text.find('<td', end)
        if start < end: break
        end = text.find('>', start)
        if end < start: break
        start = text.find('</td>', end)
        if start < end: break
        attrs['note'] = text[end+1:start].replace("\n", "").strip()
        # Nombre de rponses
        start = text.find('<td', end)
        if start < end: break
        end = text.find('>', start)
        if end < start: break
        start = text.find('</td>', end)
        if start < end: break
        attrs['status'] = text[end+1:start].replace("\n", "").strip()
        # Date dernire rponse
        end = text.find('<td>', start)
        if end < start: break
        start = text.find('</td>', end)
        if start < end: break
        attrs['date'] = text[end+4:start].replace("\n", "").strip()
        newitem = MsgItem("Commentaires", attrs)
        newitem.read = self.isRead(newitem)
        tmplist.append(newitem)
        start = text.find('<tr>', end)
    tmplist.reverse()
    return tmplist
  
  def parse(self, file):
    text = toUnicode(file.read())
    xp, reste, votes = self.parse_votes(text)
    tmplist = self.parse_msg(text)
    tmplist += self.parse_sentmsg(text)
    tmplist += self.parse_comments(text)
    return (xp, reste, votes), tmplist
  
  def process(self, results):
    infos, items = results
    oldids = [id for id in self.objects.keys()]
    # oldvotes = "".join([item.getNote() for item in self.objects])
    self.objects = {}
    self.lastId = None
    self.firstId = None
    for msgitem in items:
      if self.firstId is None:
        self.firstId = msgitem.id
      if msgitem.id not in oldids:
        log.debug("%s: nouveau msgitem: %s" % (self.name, msgitem.id))
        self.bouchot.setState(STATE_NEW)
      if self.lastId is not None:
        self.objects[self.lastId].next = msgitem
      self.lastId = msgitem.id
      self.objects[msgitem.id] = msgitem
    notes = [int(item.getNote()) for item in self.objects.values() if item.getNote() != ""]
    if len(notes):
      moy = (0.0 + sum(notes)) / len(notes)
    else:
      moy = 0
    txt = "%sVotes: %s/%s" % infos + " - Moy.Cmt: %.2f" % moy
    self.led.setLabel(txt)
    if self.viewer_open:
      self.viewer.update()
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
  
  def isRead(self, item):
    if item.type == "Messages reus":
      return item.attributes['status'] == "lu"
    if item.type == "Commentaires":
      return item.attributes['status'].find('ouveau') <= 0
  
