# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module traducteur
##   Ce module dfinit le plug-in Traducteur
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from coinlib.persistants import Identity

import tkMessageBox
import re, urllib

def build_plugins(pycc):
  return { \
    'Traducteur': Traducteur(pycc, "Traducteur", "Un bot-interprete")
  }

"""
class TraducOptions(OptionSet):
  
  def setName(self, name):
    self.name = "Traducteur"
  
  def setObjects(self):
    self._objects = {
      'tr_id':      Option("Identit pour poster", value="", type=TYPE_CHOICE),
      'tr_plugins': Option("Plugins de traduction", 
        [ 'morseencode', 'morsedecode', 'cowboyz', 'semaphoreencode', 'semaphoredecode', 'plopifier', 'pikachize', 'sms' ],
        TYPE_LIST),
      'toedit': Option("Editables", { \
          "Gnral": [ 'tr_id', 'tr_plugins' ]
        } )
    }
"""

class TraducEditor(BotOptionsEditor):
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    self.widgets[name] = ListBox(fr, option.value, width=30, height=10)
    self.widgets[name].pack(fill=X)
    # AFR: Ajout/suppression
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'tr_plugins':
        self.object.set(opt, widget.getAll())
      else:
        self.object.set(opt, widget.get())
    self.object.save()

class TraducDialog(BotOptionsDialog):
  
  def __init__(self, title, parent, object, extra_arg):
    self.object = object
    self.plugin = extra_arg
    CoinDialog.__init__(self, parent, title, ["Lancer", "Stopper", "Trad.Palmi", "Fermer"])
  
  def bind_button(self, button):
    if button == 'Trad.Palmi':
      return self.trad_palmi
    else:
      return BotOptionsDialog.bind_button(self, button)
  
  def trad_palmi(self):
    lang = self.w_lang.get()
    if self.plugin.pycc.palmi.palmi.selection_present():
      text = toUnicode(self.plugin.pycc.palmi.palmi.selection_get())
    else:
      ptxt = self.plugin.pycc.palmi.get()
      if len(ptxt) == 0: return
      text = ptxt[:-1]
    trad = self.plugin.traduction(lang, text)
    try:
      res = toUnicode(trad)
    except:
      res = trad
    if res:
      self.plugin.pycc.palmi.palmi.inserter(res, del_selection=1)
    else:
      tkMessageBox.showinfo("pyCoinCoin - %s" % self.plugin.name, self.plugin.error, parent=self.pycc.root)
  
  def body(self, master):
    self.w_options = TraducEditor(master, self.object, extra_arg=self.plugin)
    self.w_options.pack()
    self.w_lang = ComboField(master, "Langues", self.plugin.babeltr)
    self.w_lang.pack()

class Traducteur(BotPlugin):
  
  def init(self):
    self.regexp = re.compile("[^<]*[<]\s+([_a-z]*)\s+(.*)$")
    self.babeltr = [ "zh_en", "zt_en", "en_zh", "en_zt", "en_nl", "en_fr", 
                     "en_de", "en_el", "en_it", "en_ja", "en_ko", "en_pt",
                     "en_ru", "en_es", "nl_en", "nl_fr", "fr_en", "fr_de",
                     "fr_el", "fr_it", "fr_pt", "fr_nl", "fr_es", "de_en", 
                     "de_fr", "el_en", "el_fr", "it_en", "it_fr", "ja_en",
                     "ko_en", "pt_en", "pt_fr", "ru_en", "es_en", "es_fr" ]
  
  def set_defaults(self):
    self.options.set('bot_name', self.name)
    self.options.add('tr_plugins', Option("Plugins de traduction", 
        [ 'morseencode', 'morsedecode', 'cowboyz', 'semaphoreencode', 'semaphoredecode', 'plopifier', 'pikachize', 'sms' ],
        TYPE_LIST))
    self.options.add('toedit', Option("Editables", { \
          "Gnral": [ 'bot_name', 'bot_id', 'tr_plugins' ]
        } ))
  
  def config_frame(self, master):
    return TraducEditor(master, self.options, self.pycc)
  
  def action_menu(self):
    TraducDialog(self.name, self.pycc.root, self.options, self)
  
  def bot_process(self, text):
    re_parm = self.regexp.match(text)
    if not re_parm:
      self.error = "Traducteur: invocation incorrecte (manque les langues ?)"
      log.error(self.error)
      return
    args = re_parm.groups()
    return self.traduction(args[0], args[1])
  
  def traduction(self, tr, text):
    print "### traduction invoquee", tr, repr(text)
    tr_plug = tr
    texte = text
    if tr in self.options.val('tr_plugins'):
      if tr_plug == 'sms':
        try:
          taux = int(text.split()[0])
          texte = text[len("%d"%taux)+1:]
        except:
          taux = 100
        self.pycc.plugins[tr_plug].taux = taux
    elif tr == 'help':
      return u"<b>Usage:</b> <tt>%s< langues texte</tt> avec <tt>langues</tt>  choisir parmi: %s" % (self.options.val('bot_name'), ", ".join(self.options.val('tr_plugins')+self.babeltr))
    elif tr in self.babeltr:
      tr_plug = 'babelfish'
      self.pycc.plugins[tr_plug].lang = tr
    else:
      self.error = "%s: Invocation incorrecte" % self.name
      log.warning(self.error)
      return ""
    res = self.pycc.plugins[tr_plug].text_transform(texte)
    try:
      err = self.pycc.plugins[tr_plug].error
    except:
      err = ""
    if err:
      log.warning(err)
    return res
  
